#' Data
#'
#' Data encapsulation
#'
#' @docType class
#' @importFrom R6 R6Class
#'
#' @field .data List of \itemize{
#'   \item time: vector of times
#'   \item serie: centered series
#'   \item level: corresponding levels
#'   \item exo: exogenous variables
#'   \item exo_hat: predicted exogenous variables}
#'
#' @section Methods:
#' \describe{
#' \item{\code{getSize()}}{
#'   Return number of series in dataset.}
#' \item{\code{getStdHorizon()}}{
#'   Return number of time steps from serie[1] until midnight}
#' \item{\code{append(new_time, new_centered_serie, new_level, new_exo, new_exo_hat)}}{
#'   Acquire a new vector of lists (time, centered_serie, level, exo, exo_hat).}
#' \item{\code{getTime(index)}}{
#'   Get times at specified index.}
#' \item{\code{getCenteredSerie(index)}}{
#'   Get centered serie at specified index.}
#' \item{\code{getCenteredSeries(indices)}}{
#'   Get centered series at specified indices (in columns).}
#' \item{\code{getLevel(index)}}{
#'   Get level at specified index.}
#' \item{\code{getSerie(index)}}{
#'   Get serie (centered+level) at specified index.}
#' \item{\code{getSeries(indices)}}{
#'   Get series at specified indices (in columns).}
#' \item{\code{getExo(index)}}{
#'   Get exogenous variables at specified index.}
#' \item{\code{getExoHat(index)}}{
#'   Get estimated exogenous variables at specified index.}
#' }
Data = R6::R6Class("Data",
	private = list(
		.data = list()
	),
	public = list(
		getSize = function()
			length(private$.data)
		,
		getStdHorizon = function()
			24 - as.POSIXlt( private$.data[[1]]$time[1] )$hour + 1
		,
		append = function(new_time, new_centered_serie, new_level, new_exo, new_exo_hat)
		{
			private$.data[[length(private$.data)+1]] <- list(
				"time"=new_time, "centered_serie"=new_centered_serie, "level"=new_level,
				"exo"=new_exo, "exo_hat"=new_exo_hat)
		},
		getTime = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$time
		},
		getCenteredSerie = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$centered_serie
		},
		getCenteredSeries = function(indices)
			sapply(indices, function(i) self$getCenteredSerie(i))
		,
		getLevel = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$level
		},
		getSerie = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$centered_serie + private$.data[[index]]$level
		},
		getSeries = function(indices)
			sapply(indices, function(i) self$getSerie(i))
		,
		getExo = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$exo
		},
		getExoHat = function(index)
		{
			index = dateIndexToInteger(index, self)
			private$.data[[index]]$exo_hat
		},
		removeFirst = function()
			private$.data <- private$.data[2:length(private$.data)]
		,
		removeLast = function()
			private$.data <- private$.data[1:(length(private$.data)-1)]
	)
)
